package	jp.co.sds_corp.tool.dba;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.Vector;

import jp.co.sds_corp.util.common.CmnString;

public abstract class TableGenerator
{
	String	file_name;
	String	table;
	boolean	useSqlTool = false;
	int		methodType = ColumnInfo.LOWER;
	int		columnType = ColumnInfo.LOWER;
	boolean	debug = false;
	Vector<ColumnInfo>	vecClm = new Vector<ColumnInfo>();
	String	encode;

	public TableGenerator( String file_name )
		throws Exception
	{
		this.file_name = file_name;
	}

	protected void readFile() throws Exception {
		File file = new File( file_name );
		String name = file.getName();
		if ( name.indexOf( "." ) < 0 ) {
			throw( new Exception( "Illegal file name:" + file_name ));
		}
		table = name.substring( 0, name.indexOf( "." ));
		Reader rd;
		if ( encode == null ) {
			rd = new InputStreamReader( new FileInputStream( file_name ) );
		} else {
			rd = new InputStreamReader( new FileInputStream( file_name ), encode );
		}
		BufferedReader br = new BufferedReader( rd );
		int		nLine = 0;
		String	line;
		for ( ;; ) {
			line = br.readLine();
			if ( line == null ) {
				break;
			}
			nLine++;
			// 1行目は読み飛ばし
			if ( nLine == 1 ) {
				continue;
			}
			// 空行は読み飛ばし
			if ( line.length() == 0 ) {
				continue;
			}
//			String[] buf = line.split( "\t" );
			String[] buf = CmnString.getTokens( line, "\t" );
			ColumnInfo col = new ColumnInfo( buf );
			col.setMethodType( methodType );
			col.setColumnType( columnType );
			vecClm.add( col );
			if ( isDebug()) {
				System.out.println( "column(" + nLine + ") " + col );
			}
		}
		br.close();
	}

	public void setUseSqlTool( boolean var ) {
		this.useSqlTool = var;
	}
	public boolean useSqlTool() {
		return this.useSqlTool;
	}

	public void setMethodType( int var ) {
		this.methodType = var;
	}
	public int getMethodType() {
		return this.methodType;
	}

	public void setColumnType( int var ) {
		this.columnType = var;
	}
	public int getColumnType() {
		return this.columnType;
	}

	public void setEncode( String var ) {
		this.encode = var;
	}
	public String getEncode() {
		return this.encode;
	}

	public void setDebug( boolean var ) {
		this.debug = var;
	}
	public boolean isDebug() {
		return this.debug;
	}

	public void create()
		throws Exception
	{
		readFile();
		create( table + ".tbl" );
	}

	public void create( String file )
		throws Exception
	{
		Writer wr;
		if ( encode == null ) {
			wr = new OutputStreamWriter( new FileOutputStream( file ) );
		} else {
			wr = new OutputStreamWriter( new FileOutputStream( file ), encode );
		}
		PrintWriter out = new PrintWriter( wr );
		create( out );
		out.flush();
		out.close();
	}

	public void create( PrintWriter out ) throws Exception {
		printHeader( out );
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			try {
				printColumn( out, col, i );
			} catch ( RuntimeException ex ) {
				System.out.println( "create error column(" + (i+1) + ") "
					+ col.getName() + " " + col.getType() );
				throw( ex );
			}
		}
		printFooter( out );
	}

	protected boolean isEndColumn( int nPos ) {
		return nPos == ( vecClm.size() - 1 );
	}

	protected int countKey() {
		int     nKey = 0;
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			if ( !col.isKey()) {
				continue;
			}
			nKey++;
		}
		return nKey;
	}

	protected abstract void printHeader( PrintWriter out );
	protected abstract void printColumn( PrintWriter out, ColumnInfo col, int nPos );
	protected abstract void printFooter( PrintWriter out );
}

