package	jp.co.sds_corp.tool.photolist;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeMap;

public class PhotoList
{
	String	htmlname = "index.html";
	String	title = null;
	String	srcdir = ".";
	String	thumbdir = "Thumb";
	String	ext = "png";
	int		nWidth, nHeight;
	int		nOrder = 0;
	Thumb	thumb;
	boolean	bName = true;
	boolean	bRec = true;
	String	desc = null;
	HashMap<String,String>	mapDesc = new HashMap<String,String>();

	public void init() throws Exception {
		title = System.getProperty( "thumb.title" );
		if ( title == null ) {
			title = readLine( "Title" );
		}
		String var;
		var = System.getProperty( "thumb.width" );
		if ( var != null ) {
			nWidth = Integer.valueOf( var ).intValue();
		} else {
			nWidth = 50;
		}
		nHeight = 50;
		var = System.getProperty( "thumb.order" );
		if ( var != null ) {
			if ( var.equals( "name" )) {
				nOrder = 0;
			} else if ( var.equals( "time" )) {
				nOrder = 1;
			}
		}
		var = System.getProperty( "thumb.html" );
		if ( var != null ) {
			htmlname = var;
		}

		thumb = new Thumb();
		var = System.getProperty( "thumb.ext" );
		if ( var != null ) {
			ext = var;
		}
		thumb.setExt( ext );
		var = System.getProperty( "thumb.dir" );
		if ( var != null ) {
			thumbdir = var;
		}
		//thumb.setDir( thumbdir );
		thumb.setWidth( nWidth );
		thumb.setHeight( nHeight );

		var = System.getProperty( "thumb.desc" );
		if ( var != null ) {
			desc = var;
			readDesc();
		}
		var = System.getProperty( "thumb.name" );
		if ( var != null && var.equals( "false" )) {
			bName = false;
		}

		printParam();
	}

	private void readDesc() throws Exception {
		BufferedReader br = new BufferedReader( new FileReader( desc ));
		for ( ;; ) {
			String line = br.readLine();
			if ( line == null ) {
				break;
			}
			String[] arr = line.split( "\t" );
			mapDesc.put( arr[0].toUpperCase(), arr[1] );
		}
	}

	private void printParam() {
		System.out.println( "title=" + title );
		System.out.println( "width=" + nWidth );
		System.out.println( "height=" + nHeight );
		System.out.println( "ext=" + ext );
		System.out.println( "thumbdir=" + thumbdir );
		System.out.println( "desc=" + desc );
		System.out.println( "recursive=" + bRec );
	}
	public String readLine( String msg ) throws Exception {
		System.out.print( msg + ": " );
		BufferedReader br = new BufferedReader(
								new InputStreamReader( System.in ));
		String ret = br.readLine();
		return ret;
	}

	public void start() {
		try {
			if ( bRec ) {
				listDir( srcdir );
			} else {
				indexDir( srcdir );
			}
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}

	protected void listDir( String srcdir ) throws Exception {
		indexDir( srcdir );

		File	src = new File( srcdir );
		File[]	list = src.listFiles();
		for ( int i = 0; i < list.length; i++ ) {
			if ( list[i].getName().equals( thumbdir )) {
				continue;
			}
			if ( list[i].isDirectory()) {
				listDir( list[i].getPath() );
			}
		}
	}

	protected void indexDir( String srcdir ) throws Exception {
		System.out.println( "-- create index " + srcdir );
		String td = srcdir + File.separator + thumbdir;
		thumb.setDir( td );

		new File( td ).mkdirs();
		PrintWriter out = new PrintWriter( new FileOutputStream(
								srcdir + File.separator + htmlname ));
		out.println( "<HTML>" );
		out.println( "<TITLE>" + title + "</TITLE>" );
		out.println( "<BODY>" );
		out.println( "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=0>" );
		out.print( "<TR>" );
		out.print( "<TD></TD>" );
		if ( bName ) {
			out.print( "<TD>Name</TD>" );
		}
		out.print( "<TD>Size</TD><TD>Last</TD>" );
		if ( desc != null ) {
			out.print( "<TD>Desc</TD>" );
		}
		out.println( "</TR>" );

		File	src = new File( srcdir );
		File[]	list = src.listFiles();
		TreeMap<String,File>	mapFile = new TreeMap<String,File>();
		for ( int i = 0; i < list.length; i++ ) {
			String key;
			if ( nOrder == 1 ) {
				key = String.valueOf( list[i].lastModified() );
			} else {
				key = list[i].getName();
			}
			mapFile.put( key, list[i] );
		}

		Iterator<String> itr = mapFile.keySet().iterator();
		while ( itr.hasNext()) {
			String key = (String )itr.next();
			File f = (File )mapFile.get( key );
			System.out.print( f.getPath() + " ... " );
			if ( f.getName().equals( thumbdir ) ||
				f.getName().equals( htmlname ))
			{
				System.out.println( "ignore." );
				continue;
			}

			out.print( "<TR>" + "<TD>" );
			if ( f.isDirectory()) {
				if ( !bName ) {
					out.print( "<A HREF='" + f.getName() + "'>" );
					out.print( f.getName() + "</A>" );
				}
			} else {
				System.out.print( "thumbnail " );
				boolean bImg = false;
				try {
					bImg = thumb.createThumb( f );
				} catch ( Exception e ) {
				}
				if ( bImg ) {
					out.print( "<A HREF='" + f.getName() + "'>" );
					out.print( "<IMG BORDER=0 SRC='"
						+ thumbdir + "/" + thumb.getResultName() + "'>" );
					out.print( "</A>" );
				} else {
					out.print( "<A HREF='" + f.getName() + "'>" );
					out.print( f.getName() + "</A>" );
				}
			}
			out.println( "</TD>" );
			if ( bName ) {
				out.println( "<TD><A HREF='" + f.getName() + "'>"
						+ f.getName() + "</A></TD>" );
			}
			String len = null;
			if ( f.isDirectory()) {
				System.out.print( "diretory " );
				out.println( "<TD>&lt;dir&gt;</TD>" );
			} else {
				len = getLengthString( f.length() );
				out.println( "<TD ALIGN='right'>" + len + "</TD>" );
			}
			long lLast = f.lastModified();
			Date dt = new Date( lLast );
			SimpleDateFormat dtf = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss" );
			out.println( "<TD>" + dtf.format( dt ) + "</TD>" );
			String s = null;
			if ( desc != null ) {
				s = (String )mapDesc.get( f.getName().toUpperCase());
				if ( s == null ) {
					s = "---";
				}
				out.println( "<TD>" + s +  "</TD>" );
			}
			out.println( "</TR>" );
			System.out.println( "done. "  + len);
		}
		out.println( "</TABLE>" );
		out.println( "</BODY>" );
		out.println( "</HTML>" );
		out.flush();
		out.close();
	}

	private String getLengthString( long l ) {
		String unit = "KMGT";
		int		u = 0;
		double	or = l;
		double	r = l;
		for ( ;; ) {
			if ( r < 1000 ) {
				break;
			}
			or = r;
			r = r / 1024;
			u++;
		}
		DecimalFormat df = new DecimalFormat( "#,###" );
		String ret = null;
		if ( u == 0 ) {
			ret = df.format( r );
		} else {
			if ( r < 10 ) {
				df = new DecimalFormat( "0.0#" );
			}
			ret = df.format( r ) + unit.charAt( u - 1 );
		}
		return ret;
	}

	public static void main( String[] args ) {
		try {
			PhotoList obj = new PhotoList();
			obj.init();
			obj.start();
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}
}
